@@ -19,51 +19,53 @@
 #' data(germancredit)
 #'
 #' # variable filter
-#' dt_sel <- var_filter(germancredit, y = "creditability")
+#' dt_sel = var_filter(germancredit, y = "creditability")
 #'
 #'
 #' @import data.table
 #' @export
 #'
-var_filter <- function(dt, y, x = NULL, iv_limit = 0.02, na_perc_limit = 0.95, ele_perc_limit = 0.95, var_rm = NULL, var_kp = NULL, return_rm_reason = FALSE, positive="bad|1") {
+var_filter = function(dt, y, x = NULL, iv_limit = 0.02, na_perc_limit = 0.95, ele_perc_limit = 0.95, var_rm = NULL, var_kp = NULL, return_rm_reason = FALSE, positive="bad|1") {
   . = info_value = variable = rt = rm_reason = NULL # no visible binding for global variable
 
   # set dt as data.table
-  dt <- setDT(dt)
+  dt = setDT(dt)
+  # remove date/time col
+  dt = rm_datetime_col(dt)
   # replace "" by NA
-  dt <- rep_blank_na(dt)
+  dt = rep_blank_na(dt)
   # check y
-  dt <- check_y(dt, y, positive)
+  dt = check_y(dt, y, positive)
   # x variable names
-  x <- x_variable(dt,y,x)
+  x = x_variable(dt,y,x)
 
   # force removed variables
-  if (!is.null(var_rm))  x <- setdiff(x, var_rm)
+  if (!is.null(var_rm))  x = setdiff(x, var_rm)
 
   # -iv
-  iv_list <- iv(dt, y, x)
+  iv_list = iv(dt, y, x)
   # -na percentage
-  na_perc <- dt[, sapply(.SD, function(a) sum(is.na(a))/length(a)), .SDcols = x]
+  na_perc = dt[, sapply(.SD, function(a) sum(is.na(a))/length(a)), .SDcols = x]
   # -element percentage
-  ele_perc <- dt[, sapply(.SD, function(a) max(table(a)/sum(!is.na(a)))), .SDcols = x]
+  ele_perc = dt[, sapply(.SD, function(a) max(table(a)/sum(!is.na(a)))), .SDcols = x]
 
   # datatable  iv na ele
-  dt_var_selector <-
+  dt_var_selector =
     iv_list[data.table(variable = names(na_perc), na_perc = na_perc), on="variable"
     ][data.table(variable = names(ele_perc), ele_perc = ele_perc), on="variable"]
 
   # remove na_perc>95 | ele_perc>0.95 | iv<0.02
   # variable datatable selected
-  dt_var_sel <- dt_var_selector[info_value >= iv_limit & na_perc <= na_perc_limit & ele_perc <= ele_perc_limit]
+  dt_var_sel = dt_var_selector[info_value >= iv_limit & na_perc <= na_perc_limit & ele_perc <= ele_perc_limit]
 
   # add kept variable
-  x_selected <- dt_var_sel[, as.character(variable)]
-  if (!is.null(var_kp))  x_selected <- unique(c(x_selected, var_kp))
+  x_selected = dt_var_sel[, as.character(variable)]
+  if (!is.null(var_kp))  x_selected = unique(c(x_selected, var_kp))
 
 
   if (return_rm_reason) {
     # variable datatable deleted
-    dt_var_rm <- dt_var_selector[
+    dt_var_rm = dt_var_selector[
       info_value < iv_limit | na_perc > na_perc_limit | ele_perc > ele_perc_limit
     ][, `:=`(
       info_value = ifelse(info_value < iv_limit, paste0("iv < ", iv_limit), ""),
@@ -71,26 +73,26 @@ var_filter <- function(dt, y, x = NULL, iv_limit = 0.02, na_perc_limit = 0.95, e
       ele_perc = ifelse(ele_perc > ele_perc_limit, paste0("identical rate > ", ele_perc_limit), "")
     )][]
 
-    dt_rm_reason <- melt(
+    dt_rm_reason = melt(
       dt_var_rm, id.vars = "variable", variable.name="var", value.name="rm_reason", variable.factor=TRUE
     )[rm_reason != ""][
       ,.(rm_reason=paste0(rm_reason, collapse=",")), by="variable"]
 
     if (!is.null(var_rm)) {
-      dt_rm_reason <- rbind(
+      dt_rm_reason = rbind(
         dt_rm_reason,
         data.table(variable=var_rm, rm_reason="force remove")
       )
     }
     if (!is.null(var_kp)) {
-      dt_rm_reason <- dt_rm_reason[!(variable %in% var_kp)]
+      dt_rm_reason = dt_rm_reason[!(variable %in% var_kp)]
     }
 
     # return
-    rt$dt <- dt[, c(x_selected, y), with=FALSE ]
-    rt$rm <- dt_rm_reason
+    rt$dt = dt[, c(x_selected, y), with=FALSE ]
+    rt$rm = dt_rm_reason
   } else {
-    rt <- dt[, c(x_selected, y), with=FALSE ]
+    rt = dt[, c(x_selected, y), with=FALSE ]
   }
   return(rt)
 }
